package com.sample;

import java.util.HashSet;
import java.util.Set;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.KnowledgePackage;
import org.drools.definition.rule.Rule;
import org.drools.event.rule.ActivationCancelledEvent;
import org.drools.event.rule.ActivationCreatedEvent;
import org.drools.event.rule.DefaultAgendaEventListener;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

/**
 * This is a sample class to launch a rule.
 */
public class DroolsTest {

	public static final void main(String[] args) {
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase
					.newStatefulKnowledgeSession();

			Set<Rule> rules = new HashSet<Rule>();
			Set<String> ruleNames = new HashSet<String>();
			for (KnowledgePackage pkge : kbase.getKnowledgePackages()) {
				for (Rule rule : pkge.getRules()) {
					rules.add(rule);
					ruleNames.add(rule.getName());
				}
			}

			MyAgendaEventListener listener = new MyAgendaEventListener(rules,
					ruleNames);
			ksession.addEventListener(listener);

			Message message = new Message();
			message.setMessage("Hello World");
			message.setStatus(Message.HELLO);

			displayRules("Rules not executed", ruleNames);

			ksession.insert(message);
			ksession.fireAllRules();
			
			displayRules("Rules not executed", ruleNames);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("Sample.drl"),
				ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error : errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	private static void displayRules(String message, Set<String> ruleNames) {
		System.out.println(message);
		for (String ruleName : ruleNames) {
			System.out.println("- " + ruleName);
		}
	}

	public static class Message {

		public static final int HELLO = 0;
		public static final int GOODBYE = 1;
		public static final int DUFF = 2;

		private String message;

		private int status;

		public String getMessage() {
			return this.message;
		}

		public void setMessage(String message) {
			this.message = message;
		}

		public int getStatus() {
			return this.status;
		}

		public void setStatus(int status) {
			this.status = status;
		}

	}

	public static class MyAgendaEventListener extends
			DefaultAgendaEventListener {

		private Set<Rule> rules;
		private Set<String> ruleNames;

		public MyAgendaEventListener(Set<Rule> rules, Set<String> ruleNames) {
			this.rules = rules;
			this.ruleNames = ruleNames;
		}

		@Override
		public void activationCancelled(ActivationCancelledEvent event) {
			final Rule rule = event.getActivation().getRule();
			final String ruleName = rule.getName();
			System.out.println("Activation cancelled --> " + ruleName);
			rules.add(rule);
			ruleNames.add(ruleName);
		}

		@Override
		public void activationCreated(ActivationCreatedEvent event) {
			final Rule rule = event.getActivation().getRule();
			final String ruleName = rule.getName();
			System.out.println("Activation created --> " + ruleName);
			rules.remove(rule);
			ruleNames.remove(ruleName);
		}

	}
}